#!/usr/bin/perl

#--=agg=-- version 0.2 requires gfeed >= 2.3.3
#Copyright 2004, 2005 Thomas Coppi
#Dual liscenced under the GNU GPL and the artistic liscence(same terms as gfeed itself)

#Reads a config file specified on the command line and uses gfeed's command line output to aggregate interesting feeds together 

#use warnings; #enable when debugging

sub process();
sub get_feed_titles();
sub fish_titles_descriptions_links();
sub output_feed();
sub sort_rand();
sub write_tile_desc_link;

my $title_index = 0;
my $searchdesc = 0;
my $output;
my @feed_configs;
my @feed_regex;
my @titles;
my @descriptions;
my @links;

open(FILE, $ARGV[0]) or die("Can't open agg configuration file.");

#get all of the config files into an array and parse other options too
print "Reading config file...\n";
while(<FILE>) {
	chop;
	%pair = split /:/;
	process();
}

get_feed_titles();

#split up the %pair hash into arrays
sub process()
{
	@key = keys(%pair);
	@tmp = @key;
	$number = chop(@tmp);
	
	if($key[0] =~ /regex/) {
		$feed_regex[$number] = $pair{$key[0]};
#print $feed_regex[$number];
	}
	elsif($key[0] =~ /output/) {
		$output = $pair{$key[0]};
	}
	elsif($key[0] =~ /title/) {
		$title = $pair{$key[0]};
	}
	elsif($key[0] =~ /searchdesc/) {
		$searchdesc = $pair{$key[0]};
	}
	else {
		$feed_configs[$number] = $pair{$key[0]};
#print $feed_configs[$number];
	}	
	
}

#for each config file, get the titles of the items in the feed
sub get_feed_titles()
{
	our $i = -1;
	
	print "Parsing feeds";
	foreach $feed (@feed_configs) {
		$i++;
		my $gfeed_command = "gfeed $feed -tdl";
		@raw_titles = `$gfeed_command`;
		print ".";
		fish_titles_descriptions_links();		
	}
	print ".\n";
	sort_rand();
	output_feed();
}

sub fish_titles_descriptions_links()
{
	#start from the third line, because the first two lines are status
	for($j=2;$j<@raw_titles;$j++) {	
		@tmp = split(/-/,$raw_titles[$j],2);
		if($tmp[0] eq 'Title') { #just to make sure
			print ".";
			#now we check to see if there is anything interesting(matches the regex...)
			if($tmp[1] =~ $feed_regex[$i]) { #$i is from get_feed_titles() - I know its sloppy, but this is perl
				#now we can split out the number and put it in the @titles array
				@temp = split(/:/,$tmp[1],2);
				$titles[$title_index] = $temp[1];
				chop $titles[$title_index];
#print $titles[$title_index];
				#since the title matches the regex, we can take the description too...
				@tmp = split(/-/,$raw_titles[++$j],2);
				@temp = split(/:/,$tmp[1],2);
				$descriptions[$title_index] = $temp[1];
#print $descriptions[$title_index];
				#and also the link...
				@tmp = split(/-/,$raw_titles[++$j],2);	
				if(!defined $tmp[1]) { #we could try the next line...
					@tmp = split(/-/,$raw_titles[++$j],2);
					if(!defined $tmp[1]) { #if its still not there, forget it
				       		$j--;
						$temp[1] = '';
				 	}		
				}
				@temp = split(/:/,$tmp[1],2);
				$links[$title_index++] = $temp[1];
				print ".";
#print $links[$title_index-1];
			}	
		}
		elsif(($tmp[0] eq 'Description') and ($searchdesc)) { #we get to search the description
			print ".";
			if($tmp[1] =~ $feed_regex[$i]) { #$i if from get_feed_titles()
				#take the description
				@temp = split(/:/,$tmp[1],2);
				$descriptions[$title_index] = $temp[1];

				#now we have to take the title
				@tmp = split(/-/,$raw_titles[--$j],2);
				@temp = split(/:/,$tmp[1],2);
				$titles[$title_index] = $temp[1];
				chop $titles[$title_index];

				#and also the link...
				$j++;
				@tmp = split(/-/,$raw_titles[++$j],2);
				if(!defined $tmp[1]) { #we could try the next line...
					@tmp = split(/-/,$raw_titles[++$j],2);
					if(!defined $tmp[1]) { #forget it
						$j--;
						$temp[1] = '';
					}
				}
				@temp = split(/:/,$tmp[1],2);
				$links[$title_index++] = $temp[1];
				print ".";
			}
		}
	}	
}

#output the feed..
#there isn't any need for a fancy XML module, since all were are doing is writing the same thing over and over
#XML libs are better for reading XML anyway...
sub output_feed()
{
	print "Outputing";
	open(OUTPUT,">$output") or die("Can't write output file $output");

	print OUTPUT "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";
	print OUTPUT "<rss version=\"2.0\">\n";
	print OUTPUT "<channel>\n";
	print OUTPUT "<title>$title</title>\n";
	print OUTPUT "<description>\nThis RSS feed has been automatically generated by the agg Perl script</description>\n";
	print OUTPUT "<link>\nhttp://sourceforge.net/projects/gfeed\n</link>\n";
	print ".";
	
	for($i=0;$i<$title_index;$i++) {
		write_title_desc_link($titles[$i],$descriptions[$i],$links[$i]);
	}
	print OUTPUT "</channel>\n</rss>";
	print ".\nDone";
	
}

sub write_title_desc_link
{
	$_[0] =~ s/&/&amp;/g;
	$_[1] =~ s/&/&amp;/g;
	$_[2] =~ s/&/&amp;/g;
	
	print ".";
	#sigh... OUTPUT comes from output_feed... I know its sloppy, but this is perl we're talking about. It needs to be easy.
	print OUTPUT "<item>\n";
	print OUTPUT "<title>$_[0]</title>\n";
	print OUTPUT "<link>\n$_[2]</link>\n";
	print OUTPUT "<description>\n$_[1]</description>\n";
	print OUTPUT "</item>\n";
}

sub sort_rand()
{
	print "Sorting";
		
	#attempt to randomly move around the items a bit(not really sorting...)
	for(my $i=0;$i<$title_index;$i++) {
		$temp = int(rand($title_index));
		
		$tmp = $titles[$i];
		$titles[$i] = $titles[$temp];
		$titles[$temp] = $tmp;

		$tmp = $descriptions[$i];
		$descriptions[$i] = $descriptions[$temp];
		$descriptions[$temp] = $tmp;

		$tmp = $links[$i];
		$links[$i] = $links[$temp];
		$links[$temp] = $tmp;
		print ".";
	}
	print ".\n";
}

print "\n";
0;
